@@ -3,17 +3,17 @@ package Test::Nginx::Util;
 use strict;
 use warnings;
 
-our $VERSION = '0.09';
+our $VERSION = '0.10';
 
 use base 'Exporter';
 
 use POSIX qw( SIGQUIT SIGKILL SIGTERM );
 use File::Spec ();
 use HTTP::Response;
-use Module::Install::Can;
 use Cwd qw( cwd );
 use List::Util qw( shuffle );
 use Time::HiRes qw( sleep );
+use ExtUtils::MakeMaker ();
 
 our $LatestNginxVersion = 0.008039;
 
@@ -23,7 +23,7 @@ our $Profiling = 0;
 our $RepeatEach = 1;
 our $MAX_PROCESSES = 10;
 
-our $NoShuffle = 0;
+our $NoShuffle = $ENV{TEST_NGINX_NO_SHUFFLE} || 0;
 
 our $UseValgrind = $ENV{TEST_NGINX_USE_VALGRIND};
 
@@ -41,16 +41,16 @@ if ($Profiling || $UseValgrind) {
     $ForkManager = new Parallel::ForkManager($MAX_PROCESSES);
 }
 
+our $NginxBinary            = $ENV{TEST_NGINX_BINARY} || 'nginx';
 our $Workers                = 1;
 our $WorkerConnections      = 64;
-our $LogLevel               = 'debug';
-our $MasterProcessEnabled   = 'off';
+our $LogLevel               = $ENV{TEST_NGINX_LOG_LEVEL} || 'debug';
+our $MasterProcessEnabled   = $ENV{TEST_NGINX_MASTER_PROCESS} || 'off';
 our $DaemonEnabled          = 'on';
-our $ServerPort             = 1984;
-our $ServerPortForClient    = $ENV{TEST_NGINX_CLIENT_PORT} || 1984;
-our $NoRootLocation = 0;
-#our $ServerPortForClient    = 1984;
-
+our $ServerPort             = $ENV{TEST_NGINX_SERVER_PORT} || $ENV{TEST_NGINX_PORT} || 1984;
+our $ServerPortForClient    = $ENV{TEST_NGINX_CLIENT_PORT} || $ENV{TEST_NGINX_PORT} || 1984;
+our $NoRootLocation         = 0;
+our $TestNginxSleep         = $ENV{TEST_NGINX_SLEEP} || 0;
 
 sub repeat_each (@) {
     if (@_) {
@@ -128,6 +128,8 @@ our @EXPORT_OK = qw(
     log_level
     no_shuffle
     no_root_location
+    html_dir
+    server_root
 );
 
 
@@ -150,7 +152,7 @@ our $TODO;
 
 #our ($PrevRequest, $PrevConfig);
 
-our $ServRoot   = File::Spec->catfile(cwd(), 't/servroot');
+our $ServRoot   = $ENV{TEST_NGINX_SERVROOT} || File::Spec->catfile(cwd(), 't/servroot');
 our $LogDir     = File::Spec->catfile($ServRoot, 'logs');
 our $ErrLogFile = File::Spec->catfile($LogDir, 'error.log');
 our $AccLogFile = File::Spec->catfile($LogDir, 'access.log');
@@ -159,6 +161,14 @@ our $ConfDir    = File::Spec->catfile($ServRoot, 'conf');
 our $ConfFile   = File::Spec->catfile($ConfDir, 'nginx.conf');
 our $PidFile    = File::Spec->catfile($LogDir, 'nginx.pid');
 
+sub html_dir () {
+    return $HtmlDir;
+}
+
+sub server_root () {
+    return $ServRoot;
+}
+
 sub run_tests () {
     $NginxVersion = get_nginx_version();
 
@@ -179,11 +189,18 @@ sub run_tests () {
 
 sub setup_server_root () {
     if (-d $ServRoot) {
-        #sleep 0.5;
-        #die ".pid file $PidFile exists.\n";
-        system("rm -rf t/servroot > /dev/null") == 0 or
-            die "Can't remove t/servroot";
-        #sleep 0.5;
+        # Take special care, so we won't accidentally remove
+        # real user data when TEST_NGINX_SERVROOT is mis-used.
+        system("rm -rf $ConfDir > /dev/null") == 0 or
+            die "Can't remove $ConfDir";
+        system("rm -rf $HtmlDir > /dev/null") == 0 or
+            die "Can't remove $HtmlDir";
+        system("rm -rf $LogDir > /dev/null") == 0 or
+            die "Can't remove $LogDir";
+        system("rm -rf $ServRoot/*_temp > /dev/null") == 0 or
+            die "Can't remove $ServRoot/*_temp";
+        system("rmdir $ServRoot > /dev/null") == 0 or
+            die "Can't remove $ServRoot (not empty?)";
     }
     mkdir $ServRoot or
         die "Failed to do mkdir $ServRoot\n";
@@ -250,8 +267,8 @@ sub write_user_files ($) {
     }
 }
 
-sub write_config_file ($$) {
-    my ($config, $http_config) = @_;
+sub write_config_file ($$$) {
+    my ($config, $http_config, $main_config) = @_;
 
     if (!defined $config) {
         $config = '';
@@ -261,6 +278,10 @@ sub write_config_file ($$) {
         $http_config = '';
     }
 
+    if (!defined $main_config) {
+        $main_config = '';
+    }
+
     open my $out, ">$ConfFile" or
         die "Can't open $ConfFile for writing: $!\n";
     print $out <<_EOC_;
@@ -270,6 +291,8 @@ master_process $MasterProcessEnabled;
 error_log $ErrLogFile $LogLevel;
 pid       $PidFile;
 
+$main_config
+
 http {
     access_log $AccLogFile;
 
@@ -321,11 +344,11 @@ sub get_canon_version (@) {
 }
 
 sub get_nginx_version () {
-    my $out = `nginx -V 2>&1`;
+    my $out = `$NginxBinary -V 2>&1`;
     if (!defined $out || $? != 0) {
         warn "Failed to get the version of the Nginx in PATH.\n";
     }
-    if ($out =~ m{nginx/(\d+)\.(\d+)\.(\d+)}s) {
+    if ($out =~ m{(?:nginx|ngx_openresty)/(\d+)\.(\d+)\.(\d+)}s) {
         $NginxRawVersion = "$1.$2.$3";
         return get_canon_version($1, $2, $3);
     }
@@ -340,7 +363,7 @@ sub get_pid_from_pidfile ($) {
     my $pid = do { local $/; <$in> };
     #warn "Pid: $pid\n";
     close $in;
-    $pid;
+    return $pid;
 }
 
 sub trim ($) {
@@ -485,7 +508,7 @@ sub run_test ($) {
 
             if (system("ps $pid > /dev/null") == 0) {
                 #warn "found running nginx...";
-                write_config_file($config, $block->http_config);
+                write_config_file($config, $block->http_config, $block->main_config);
                 if (kill(SIGQUIT, $pid) == 0) { # send quit signal
                     #warn("$name - Failed to send quit signal to the nginx process with PID $pid");
                 }
@@ -513,8 +536,8 @@ start_nginx:
             #warn "*** Restarting the nginx server...\n";
             setup_server_root();
             write_user_files($block);
-            write_config_file($config, $block->http_config);
-            if ( ! Module::Install::Can->can_run('nginx') ) {
+            write_config_file($config, $block->http_config, $block->main_config);
+            if ( ! can_run($NginxBinary) ) {
                 Test::More::BAIL_OUT("$name - Cannot find the nginx executable in the PATH environment");
                 die;
             }
@@ -528,9 +551,9 @@ start_nginx:
 
             my $cmd;
             if ($NginxVersion >= 0.007053) {
-                $cmd = "nginx -p $ServRoot/ -c $ConfFile > /dev/null";
+                $cmd = "$NginxBinary -p $ServRoot/ -c $ConfFile > /dev/null";
             } else {
-                $cmd = "nginx -c $ConfFile > /dev/null";
+                $cmd = "$NginxBinary -c $ConfFile > /dev/null";
             }
 
             if ($UseValgrind) {
@@ -564,7 +587,11 @@ start_nginx:
 
                 }
                 #warn "sleeping";
-                sleep 1;
+                if ($TestNginxSleep) {
+                    sleep $TestNginxSleep;
+                } else {
+                    sleep 1;
+                }
             } else {
                 if (system($cmd) != 0) {
                     Test::More::BAIL_OUT("$name - Cannot start nginx using command \"$cmd\".");
@@ -601,17 +628,33 @@ start_nginx:
         }
     }
 
+    if (my $total_errlog = $ENV{TEST_NGINX_ERROR_LOG}) {
+        my $errlog = "$LogDir/error.log";
+        if (-s $errlog) {
+            open my $out, ">>$total_errlog" or
+                die "Failed to append test case title to $total_errlog: $!\n";
+            print $out "\n=== $0 $name\n";
+            close $out;
+            system("cat $errlog >> $total_errlog") == 0 or
+                die "Failed to append $errlog to $total_errlog. Abort.\n";
+        }
+    }
+
     if ($Profiling || $UseValgrind) {
         #warn "Found quit...";
         if (-f $PidFile) {
             #warn "found pid file...";
             my $pid = get_pid_from_pidfile($name);
             if (system("ps $pid > /dev/null") == 0) {
-                write_config_file($config, $block->http_config);
+                write_config_file($config, $block->http_config, $block->main_config);
                 if (kill(SIGQUIT, $pid) == 0) { # send quit signal
                     warn("$name - Failed to send quit signal to the nginx process with PID $pid");
                 }
-                sleep 0.1;
+                if ($TestNginxSleep) {
+                    sleep $TestNginxSleep;
+                } else {
+                    sleep 0.1;
+                }
                 if (-f $PidFile) {
                     #warn "killing with force (valgrind or profile)...\n";
                     kill(SIGKILL, $pid);
@@ -630,14 +673,21 @@ start_nginx:
 }
 
 END {
-    if ($UseValgrind) {
+    if ($UseValgrind || !$ENV{TEST_NGINX_NO_CLEAN}) {
         if (-f $PidFile) {
             my $pid = get_pid_from_pidfile('');
+            if (!$pid) {
+                die "No pid found.";
+            }
             if (system("ps $pid > /dev/null") == 0) {
                 if (kill(SIGQUIT, $pid) == 0) { # send quit signal
                     #warn("$name - Failed to send quit signal to the nginx process with PID $pid");
                 }
-                sleep 0.02;
+                if ($TestNginxSleep) {
+                    sleep $TestNginxSleep;
+                } else {
+                    sleep 0.02;
+                }
                 if (system("ps $pid > /dev/null") == 0) {
                     #warn "killing with force...\n";
                     kill(SIGKILL, $pid);
@@ -650,4 +700,20 @@ END {
     }
 }
 
+# check if we can run some command
+sub can_run {
+	my ($cmd) = @_;
+
+	my $_cmd = $cmd;
+	return $_cmd if (-x $_cmd or $_cmd = MM->maybe_command($_cmd));
+
+	for my $dir ((split /$Config::Config{path_sep}/, $ENV{PATH}), '.') {
+		next if $dir eq '';
+		my $abs = File::Spec->catfile($dir, $_[1]);
+		return $abs if (-x $abs or $abs = MM->maybe_command($abs));
+	}
+
+	return;
+}
+
 1;
